==> voice-android-3.1.1-sources.jar: com/twilio/voice/LocalAudioTrack.java <==
package com.twilio.voice;

import android.content.Context;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import static android.Manifest.permission.RECORD_AUDIO;

/**
 * Represents a local audio source.
 */
class LocalAudioTrack extends AudioTrack {
    private static final Logger logger = Logger.getLogger(LocalAudioTrack.class);

    private final String trackId;
    private final MediaFactory mediaFactory;
    private long nativeLocalAudioTrackHandle;

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context An Android context.
     * @param enabled initial state of audio track.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled) {
        return create(context, enabled, null, null);
    }

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context applicatoin context.
     * @param enabled initial state of audio track.
     * @param audioOptions audio options to be applied to the track.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled,
                                         @Nullable AudioOptions audioOptions) {
        return create(context, enabled, audioOptions, null);
    }

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context An Android context.
     * @param enabled initial state of audio track.
     * @param name audio track name.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled,
                                         @Nullable String name) {
        return create(context, enabled, null, name);
    }

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context An Android context.
     * @param enabled initial state of audio track.
     * @param audioOptions audio options to be applied to track.
     * @param name audio track name.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled,
                                         @Nullable AudioOptions audioOptions,
                                         @Nullable String name) {
        Preconditions.checkNotNull(context, "context must not be null");
        Preconditions.checkState(Utils.permissionGranted(context, RECORD_AUDIO), "RECORD_AUDIO " +
                "permission must be granted to create audio track");

        // Use temporary media factory owner to create local audio track
        Object temporaryMediaFactoryOwner = new Object();
        MediaFactory mediaFactory = MediaFactory.instance(temporaryMediaFactoryOwner,
                context.getApplicationContext());
        LocalAudioTrack localAudioTrack = mediaFactory
                .createAudioTrack(context.getApplicationContext(), enabled, audioOptions, name);

        if (localAudioTrack == null) {
            logger.e("Failed to create local audio track");
        }

        // Local audio track will obtain media factory instance in constructor so release ownership
        mediaFactory.release(temporaryMediaFactoryOwner);

        return localAudioTrack;
    }

    /**
     * Check if the local audio track is enabled.
     *
     * When the value is false, the local audio track is muted. When the value is true the
     * local audio track is live.
     *
     * @return true if the local audio is enabled.
     */
    @Override
    public synchronized boolean isEnabled() {
        if (!isReleased()) {
            return nativeIsEnabled(nativeLocalAudioTrackHandle);
        } else {
            logger.w("Local audio track is not enabled because it has been released");

            return false;
        }
    }

    /**
     * Returns the local audio track name. {@link #trackId} is returned if no name was specified.
     */
    @Override
    public String getName() {
        return super.getName();
    }

    /**
     * This audio track id.
     *
     * @return track id.
     */
    public String getTrackId() {
        return trackId;
    }

    /**
     * Sets the state of the local audio track. The results of this operation are signaled to other
     * Participants in the same Room. When an audio track is disabled, the audio is muted.
     *
     * @param enable the desired state of the local audio track.
     */
    public synchronized void enable(boolean enable) {
        if (!isReleased()) {
            nativeEnable(nativeLocalAudioTrackHandle, enable);
        } else {
            logger.e("Cannot enable a local audio track that has been removed");
        }
    }

    /**
     * Releases native memory owned by audio track.
     */
    public synchronized void release() {
        if (!isReleased()) {
            nativeRelease(nativeLocalAudioTrackHandle);
            nativeLocalAudioTrackHandle = 0;
            mediaFactory.release(this);
        }
    }

    LocalAudioTrack(long nativeLocalAudioTrackHandle,
                    @NonNull String trackId,
                    @NonNull String name,
                    boolean enabled,
                    Context context) {
        super(enabled, name);
        Preconditions.checkApplicationContext(context,
                "must create local audio track with application context");
        this.trackId = trackId;
        this.nativeLocalAudioTrackHandle = nativeLocalAudioTrackHandle;
        this.mediaFactory = MediaFactory.instance(this, context);
    }

    boolean isReleased() {
        return nativeLocalAudioTrackHandle == 0;
    }

    /*
     * Called by LocalParticipant at JNI level.
     */
    @SuppressWarnings("unused")
    synchronized long getNativeHandle() {
        return nativeLocalAudioTrackHandle;
    }

    private native boolean nativeIsEnabled(long nativeLocalAudioTrackHandle);
    private native void nativeEnable(long nativeLocalAudioTrackHandle, boolean enable);
    private native void nativeRelease(long nativeLocalAudioTrackHandle);
}

==> voice-android-6.1.3-sources.jar: com/twilio/voice/LocalAudioTrack.java <==
package com.twilio.voice;

import android.content.Context;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import static android.Manifest.permission.RECORD_AUDIO;

/**
 * Represents a local audio source.
 */
class LocalAudioTrack extends AudioTrack {
    private static final Logger logger = Logger.getLogger(LocalAudioTrack.class);

    private final String trackId;
    private final MediaFactory mediaFactory;
    private long nativeLocalAudioTrackHandle;

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context An Android context.
     * @param enabled initial state of audio track.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled) {
        return create(context, enabled, null, null);
    }

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context applicatoin context.
     * @param enabled initial state of audio track.
     * @param audioOptions audio options to be applied to the track.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled,
                                         @Nullable AudioOptions audioOptions) {
        return create(context, enabled, audioOptions, null);
    }

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context An Android context.
     * @param enabled initial state of audio track.
     * @param name audio track name.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled,
                                         @Nullable String name) {
        return create(context, enabled, null, name);
    }

    /**
     * Creates an audio track. Note that the RECORD_AUDIO permission must be granted
     * in order for this operation to succeed. If RECORD_AUDIO is not granted null is returned.
     *
     * @param context An Android context.
     * @param enabled initial state of audio track.
     * @param audioOptions audio options to be applied to track.
     * @param name audio track name.
     * @return local audio track if successfully added or null if audio track could not be created.
     */
    public static LocalAudioTrack create(@NonNull Context context,
                                         boolean enabled,
                                         @Nullable AudioOptions audioOptions,
                                         @Nullable String name) {
        Preconditions.checkNotNull(context, "context must not be null");
        Preconditions.checkState(Utils.permissionGranted(context, RECORD_AUDIO), "RECORD_AUDIO " +
                "permission must be granted to create audio track");

        // Use temporary media factory owner to create local audio track
        Object temporaryMediaFactoryOwner = new Object();
        MediaFactory mediaFactory = MediaFactory.instance(temporaryMediaFactoryOwner,
                context.getApplicationContext());
        LocalAudioTrack localAudioTrack = mediaFactory
                .createAudioTrack(context.getApplicationContext(), enabled, audioOptions, name);

        if (localAudioTrack == null) {
            logger.e("Failed to create local audio track");
        }

        // Local audio track will obtain media factory instance in constructor so release ownership
        mediaFactory.release(temporaryMediaFactoryOwner);

        return localAudioTrack;
    }

    /**
     * Check if the local audio track is enabled.
     *
     * When the value is false, the local audio track is muted. When the value is true the
     * local audio track is live.
     *
     * @return true if the local audio is enabled.
     */
    @Override
    public synchronized boolean isEnabled() {
        if (!isReleased()) {
            return nativeIsEnabled(nativeLocalAudioTrackHandle);
        } else {
            logger.w("Local audio track is not enabled because it has been released");

            return false;
        }
    }

    /**
     * Returns the local audio track name. {@link #trackId} is returned if no name was specified.
     */
    @Override
    public String getName() {
        return super.getName();
    }

    /**
     * This audio track id.
     *
     * @return track id.
     */
    public String getTrackId() {
        return trackId;
    }

    /**
     * Sets the state of the local audio track. The results of this operation are signaled to other
     * Participants in the same Room. When an audio track is disabled, the audio is muted.
     *
     * @param enable the desired state of the local audio track.
     */
    public synchronized void enable(boolean enable) {
        if (!isReleased()) {
            nativeEnable(nativeLocalAudioTrackHandle, enable);
        } else {
            logger.e("Cannot enable a local audio track that has been removed");
        }
    }

    /**
     * Releases native memory owned by audio track.
     */
    public synchronized void release() {
        if (!isReleased()) {
            nativeRelease(nativeLocalAudioTrackHandle);
            nativeLocalAudioTrackHandle = 0;
            mediaFactory.release(this);
        }
    }

    LocalAudioTrack(long nativeLocalAudioTrackHandle,
                    @NonNull String trackId,
                    @NonNull String name,
                    boolean enabled,
                    Context context) {
        super(enabled, name);
        Preconditions.checkApplicationContext(context,
                "must create local audio track with application context");
        this.trackId = trackId;
        this.nativeLocalAudioTrackHandle = nativeLocalAudioTrackHandle;
        this.mediaFactory = MediaFactory.instance(this, context);
    }

    boolean isReleased() {
        return nativeLocalAudioTrackHandle == 0;
    }

    /*
     * Called by LocalParticipant at JNI level.
     */
    @SuppressWarnings("unused")
    synchronized long getNativeHandle() {
        return nativeLocalAudioTrackHandle;
    }

    private native boolean nativeIsEnabled(long nativeLocalAudioTrackHandle);
    private native void nativeEnable(long nativeLocalAudioTrackHandle, boolean enable);
    private native void nativeRelease(long nativeLocalAudioTrackHandle);
}
